==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/fir/analysis/jvm/checkers/declaration/FirImplementationByDelegationWithDifferentGenericSignatureChecker.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.jvm.checkers.declaration

import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.diagnostics.reportOn
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.declaration.FirClassChecker
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.unsubstitutedScope
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.jvm.FirJvmErrors
import ksp.org.jetbrains.kotlin.fir.declarations.FirClass
import ksp.org.jetbrains.kotlin.fir.delegatedWrapperData
import ksp.org.jetbrains.kotlin.fir.resolve.scope
import ksp.org.jetbrains.kotlin.fir.scopes.CallableCopyTypeCalculator
import ksp.org.jetbrains.kotlin.fir.scopes.ProcessorAction
import ksp.org.jetbrains.kotlin.fir.scopes.processAllFunctions
import ksp.org.jetbrains.kotlin.fir.scopes.processOverriddenFunctions
import ksp.org.jetbrains.kotlin.fir.types.resolvedType
import ksp.org.jetbrains.kotlin.fir.unwrapFakeOverrides

object FirImplementationByDelegationWithDifferentGenericSignatureChecker : FirClassChecker(MppCheckerKind.Platform) {
    override fun check(declaration: FirClass, context: CheckerContext, reporter: DiagnosticReporter) {
        val classScope = declaration.unsubstitutedScope(context)
        classScope.processAllFunctions { symbol ->
            val delegatedWrapperData = symbol.delegatedWrapperData ?: return@processAllFunctions
            val wrappedGenericFunction = delegatedWrapperData.wrapped
            if (wrappedGenericFunction.typeParameters.isEmpty()) return@processAllFunctions
            val fieldScope = delegatedWrapperData.delegateField.symbol.resolvedInitializer?.resolvedType?.scope(
                context.session, context.scopeSession, CallableCopyTypeCalculator.DoNothing, null
            ) ?: return@processAllFunctions
            var reported = false
            val genericSymbolToCompare = wrappedGenericFunction.symbol.unwrapFakeOverrides()
            fieldScope.processFunctionsByName(symbol.name) { clashedSymbol ->
                if (reported || clashedSymbol.typeParameterSymbols.isNotEmpty()) return@processFunctionsByName
                fieldScope.processOverriddenFunctions(clashedSymbol) { overriddenSymbol ->
                    if (overriddenSymbol.unwrapFakeOverrides() === genericSymbolToCompare) {
                        reporter.reportOn(
                            delegatedWrapperData.delegateField.returnTypeRef.source,
                            FirJvmErrors.IMPLEMENTATION_BY_DELEGATION_WITH_DIFFERENT_GENERIC_SIGNATURE,
                            wrappedGenericFunction.symbol,
                            clashedSymbol,
                            context
                        )
                        reported = true
                        ProcessorAction.STOP
                    } else {
                        ProcessorAction.NEXT
                    }
                }
            }
        }
    }
}

==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/fir/analysis/jvm/checkers/declaration/FirImplementationByDelegationWithDifferentGenericSignatureChecker.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.jvm.checkers.declaration

import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.diagnostics.reportOn
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.declaration.FirClassChecker
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.unsubstitutedScope
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.jvm.FirJvmErrors
import ksp.org.jetbrains.kotlin.fir.declarations.FirClass
import ksp.org.jetbrains.kotlin.fir.delegatedWrapperData
import ksp.org.jetbrains.kotlin.fir.resolve.scope
import ksp.org.jetbrains.kotlin.fir.scopes.CallableCopyTypeCalculator
import ksp.org.jetbrains.kotlin.fir.scopes.ScopeFunctionRequiresPrewarm
import ksp.org.jetbrains.kotlin.fir.scopes.ProcessorAction
import ksp.org.jetbrains.kotlin.fir.scopes.processAllFunctions
import ksp.org.jetbrains.kotlin.fir.scopes.processOverriddenFunctions
import ksp.org.jetbrains.kotlin.fir.types.resolvedType
import ksp.org.jetbrains.kotlin.fir.unwrapFakeOverrides

object FirImplementationByDelegationWithDifferentGenericSignatureChecker : FirClassChecker(MppCheckerKind.Platform) {
    context(context: CheckerContext, reporter: DiagnosticReporter)
    override fun check(declaration: FirClass) {
        val classScope = declaration.unsubstitutedScope(context)
        classScope.processAllFunctions { symbol ->
            val delegatedWrapperData = symbol.delegatedWrapperData ?: return@processAllFunctions
            val wrappedGenericFunction = delegatedWrapperData.wrapped
            if (wrappedGenericFunction.typeParameters.isEmpty()) return@processAllFunctions
            val fieldScope = delegatedWrapperData.delegateField.symbol.resolvedInitializer?.resolvedType?.scope(
                context.session, context.scopeSession, CallableCopyTypeCalculator.DoNothing, null
            ) ?: return@processAllFunctions
            var reported = false
            val genericSymbolToCompare = wrappedGenericFunction.symbol.unwrapFakeOverrides()
            fieldScope.processFunctionsByName(symbol.name) { clashedSymbol ->
                if (reported || clashedSymbol.typeParameterSymbols.isNotEmpty()) return@processFunctionsByName
                @OptIn(ScopeFunctionRequiresPrewarm::class)
                fieldScope.processOverriddenFunctions(clashedSymbol) { overriddenSymbol ->
                    if (overriddenSymbol.unwrapFakeOverrides() === genericSymbolToCompare) {
                        reporter.reportOn(
                            delegatedWrapperData.delegateField.returnTypeRef.source,
                            FirJvmErrors.IMPLEMENTATION_BY_DELEGATION_WITH_DIFFERENT_GENERIC_SIGNATURE,
                            wrappedGenericFunction.symbol,
                            clashedSymbol
                        )
                        reported = true
                        ProcessorAction.STOP
                    } else {
                        ProcessorAction.NEXT
                    }
                }
            }
        }
    }
}
